@@ -2,34 +2,94 @@
 # 1) If can't connect and user doesn't want to start H2O, stop immediately
 # 2) If user does want to start H2O and running locally, attempt to bring up H2O launcher
 # 3) If user does want to start H2O, but running non-locally, print an error
-h2o.init <- function(ip = "127.0.0.1", port = 54321, startH2O = TRUE, forceDL = FALSE, Xmx = "1g", beta = FALSE, license = NULL) {
+h2o.init <- function(ip = "127.0.0.1", port = 54321, startH2O = TRUE, forceDL = FALSE, Xmx,
+                     beta = FALSE, assertion = TRUE, license = NULL, nthreads = -2, max_mem_size = NULL, min_mem_size = NULL,
+                     ice_root = NULL, strict_version_check = TRUE) {
   if(!is.character(ip)) stop("ip must be of class character")
   if(!is.numeric(port)) stop("port must be of class numeric")
   if(!is.logical(startH2O)) stop("startH2O must be of class logical")
   if(!is.logical(forceDL)) stop("forceDL must be of class logical")
-  if(!is.character(Xmx)) stop("Xmx must be of class character")
-  if(!regexpr("^[1-9][0-9]*[gGmM]$", Xmx)) stop("Xmx option must be like 1g or 1024m")
+  if(!missing(Xmx) && !is.character(Xmx)) stop("Xmx must be of class character")
+  if(!is.numeric(nthreads)) stop("nthreads must be of class numeric")
+  if(!is.null(max_mem_size) && !is.character(max_mem_size)) stop("max_mem_size must be of class character")
+  if(!is.null(min_mem_size) && !is.character(min_mem_size)) stop("min_mem_size must be of class character")
+  if(!is.null(max_mem_size) && !regexpr("^[1-9][0-9]*[gGmM]$", max_mem_size)) stop("max_mem_size option must be like 1g or 1024m")
+  if(!is.null(min_mem_size) && !regexpr("^[1-9][0-9]*[gGmM]$", min_mem_size)) stop("min_mem_size option must be like 1g or 1024m")
+  if(!missing(Xmx) && !regexpr("^[1-9][0-9]*[gGmM]$", Xmx)) stop("Xmx option must be like 1g or 1024m")
   if(!is.logical(beta)) stop("beta must be of class logical")
+  if(!is.logical(assertion)) stop("assertion must be of class logical")
   if(!is.null(license) && !is.character(license)) stop("license must be of class character")
+  if(!is.null(ice_root) && !is.character(ice_root)) stop("ice_root must be of class character")
+  if(!is.logical(strict_version_check)) stop("strict_version_check must be of class logical")
 
+  if(!missing(Xmx)) {
+    warning("Xmx is a deprecated parameter. Use `max_mem_size` and `min_mem_size` to set the memory boundaries. Using `Xmx` to set these.")
+    max_mem_size <- Xmx
+    min_mem_size <- Xmx
+  }
+
+  if (is.null(ice_root)) {
+    ice_root = tempdir()
+  }
+
+  # Disable Up.json for now, since it doesn't exist in most builds.
+  # Re-enable this sometime a few months from now.
+  # myUpURL = paste("http://", ip, ":", port, "/Up.json", sep="")
+  myUpURL = paste("http://", ip, ":", port, sep="")
   myURL = paste("http://", ip, ":", port, sep="")
-  if(!url.exists(myURL)) {
+  warnNthreads = FALSE
+  if(!url.exists(myUpURL)) {
     if(!startH2O)
       stop(paste("Cannot connect to H2O server. Please check that H2O is running at", myURL))
     else if(ip == "localhost" || ip == "127.0.0.1") {
       cat("\nH2O is not running yet, starting it now...\n")
-      .h2o.startJar(memory = Xmx, beta = beta, forceDL = forceDL, license = license)
-      count = 0; while(!url.exists(myURL) && count < 60) { Sys.sleep(1); count = count + 1 }
-      if(!url.exists(myURL)) stop("H2O failed to start, stopping execution.")
-    } else stop("Can only start H2O launcher if IP address is localhost.")
+
+      if (nthreads == -2) {
+        warnNthreads = TRUE
+        nthreads = 2
+      }
+
+      .h2o.startJar(nthreads = nthreads, max_memory = max_mem_size, min_memory = min_mem_size, beta = beta, assertion = assertion, forceDL = forceDL, license = license, ice_root = ice_root)
+
+      count = 0;
+      while(!url.exists(myURL) && (count < 60)) {
+        Sys.sleep(1);
+        count = count + 1
+      }
+
+      if (!url.exists(myURL)) {
+        stop("H2O failed to start, stopping execution.")
+      }
+    } else {
+      stop("Can only start H2O launcher if IP address is localhost.")
+    }
   }
-  cat("Successfully connected to", myURL, "\n")
+
+  cat("Successfully connected to", myURL, "\n\n")
   H2Oserver = new("H2OClient", ip = ip, port = port)
   # Sys.sleep(0.5)    # Give cluster time to come up
-  h2o.clusterInfo(H2Oserver); cat("\n")
-  
-  if((verH2O = .h2o.__version(H2Oserver)) != (verPkg = packageVersion("h2o")))
-    stop("Version mismatch! H2O is running version ", verH2O, " but R package is version ", toString(verPkg), "\n")
+  h2o.clusterInfo(H2Oserver)
+  cat("\n")
+
+  if((verH2O = .h2o.__version(H2Oserver)) != (verPkg = packageVersion("h2o"))) {
+    message = sprintf("Version mismatch! H2O is running version %s but R package is version %s", verH2O, toString(verPkg))
+    if (strict_version_check) {
+      stop(message)
+    }
+    else {
+      warning(message)
+    }
+  }
+
+  if (warnNthreads) {
+    cat("Note:  As started, H2O is limited to the CRAN default of 2 CPUs.\n")
+    cat("       Shut down and restart H2O as shown below to use all your CPUs.\n")
+    cat("           > h2o.shutdown(localH2O)\n")
+    cat("           > localH2O = h2o.init(nthreads = -1)\n")
+    cat("\n")
+  }
+
+  assign("SERVER", H2Oserver, .pkg.env)
   return(H2Oserver)
 }
 
@@ -67,9 +127,9 @@ h2o.shutdown <- function(client, prompt = TRUE) {
 
 h2o.clusterStatus <- function(client) {
   if(missing(client) || class(client) != "H2OClient") stop("client must be a H2OClient object")
+  .h2o.__checkUp(client)
   myURL = paste("http://", client@ip, ":", client@port, "/", .h2o.__PAGE_CLOUD, sep = "")
-  if(!url.exists(myURL)) stop("Cannot connect to H2O instance at ", myURL)
-  res = fromJSON(postForm(myURL, style = "POST"))
+  res = fromJSON(postForm(myURL, .params = list(quiet="true", skip_ticks="true"), style = "POST"))
   
   cat("Version:", res$version, "\n")
   cat("Cloud name:", res$cloud_name, "\n")
@@ -119,14 +179,9 @@ h2o.clusterStatus <- function(client) {
     "'localH2O', for example):\n",
     "    > localH2O = h2o.init()\n",
     "\n",
-    "For H2O package documentation, first call init() and then ask for help:\n",
-    "    > localH2O = h2o.init()\n",
+    "For H2O package documentation, ask for help:\n",
     "    > ??h2o\n",
     "\n",
-    "To stop H2O you must explicitly call shutdown (either from R, as shown\n",
-    "here, or from the Web UI):\n",
-    "    > h2o.shutdown(localH2O)\n",
-    "\n",
     "After starting H2O, you can use the Web UI at http://localhost:54321\n",
     "For more information visit http://docs.0xdata.com\n",
     "\n",
@@ -148,12 +203,35 @@ h2o.clusterStatus <- function(client) {
   }, onexit = TRUE)
 }
 
-# .onDetach <- function(libpath) {
+
+.onDetach <- function(libpath) {
+  ip    <- "127.0.0.1";
+  port  <- 54321
+  myURL <- paste("http://", ip, ":", port, sep = "")
+  if (url.exists(myURL)) {
+    tryCatch(h2o.shutdown(new("H2OClient", ip = ip, port = port), prompt = FALSE), error = function(e) {
+      msg = paste(
+        "\n",
+        "----------------------------------------------------------------------\n",
+            "\n",
+            "Could not shut down the H2O Java Process!\n",
+            "Please shutdown H2O manually by navigating to `http://localhost:54321/Shutdown`\n\n",
+            "Windows requires the shutdown of h2o before re-installing -or- updating the h2o package.\n",
+            "For more information visit http://docs.0xdata.com\n",
+            "\n",
+            "----------------------------------------------------------------------\n",
+            sep = "")
+      warning(msg)
+    })
+  }
+}
+
+#.onDetach <- function(libpath) {
 #   if(exists(".LastOriginal", mode = "function"))
 #      assign(".Last", get(".LastOriginal"), envir = .GlobalEnv)
 #   else if(exists(".Last", envir = .GlobalEnv))
 #     rm(".Last", envir = .GlobalEnv)
-# }
+#}
 
 # .onUnload <- function(libpath) {
 #   ip = "127.0.0.1"; port = 54321
@@ -161,10 +239,10 @@ h2o.clusterStatus <- function(client) {
 #   
 #   require(RCurl); require(rjson)
 #   if(.h2o.startedH2O() && url.exists(myURL))
-#     h2o.shutdown(new("H2OClient", ip=ip, port=port), FALSE)
+#     h2o.shutdown(new("H2OClient", ip=ip, port=port), prompt = FALSE)
 # }
 
-.h2o.startJar <- function(memory = "1g", beta = FALSE, forceDL = FALSE, license = NULL) {
+.h2o.startJar <- function(nthreads = -1, max_memory = NULL, min_memory = NULL, beta = FALSE, assertion = TRUE, forceDL = FALSE, license = NULL, ice_root) {
   command <- .h2o.checkJava()
 
   if (! is.null(license)) {
@@ -173,31 +251,71 @@ h2o.clusterStatus <- function(client) {
     }
   }
 
+  if (missing(ice_root)) {
+    stop("ice_root must be specified for .h2o.startJar");
+  }
+
   # Note: Logging to stdout and stderr in Windows only works for R version 3.0.2 or later!
   stdout <- .h2o.getTmpFile("stdout")
   stderr <- .h2o.getTmpFile("stderr")
   write(Sys.getpid(), .h2o.getTmpFile("pid"), append = FALSE)   # Write PID to file to track if R started H2O
   
-  # jar_file <- paste(.h2o.pkg.path, "java", "h2o.jar", sep = .Platform$file.sep)
   jar_file <- .h2o.downloadJar(overwrite = forceDL)
   jar_file <- paste('"', jar_file, '"', sep = "")
-  args <- c(paste("-Xms", memory, sep=""),
-            paste("-Xmx", memory, sep=""),
-            "-jar", jar_file,
-            "-name", "H2O_started_from_R",
-            "-ip", "127.0.0.1",
-            "-port", "54321"
-            )
+
+  # Throw an error if GNU Java is being used
+  jver <- system2(command, "-version", stdout = TRUE, stderr = TRUE)
+  if(any(grepl("GNU libgcj", jver))) {
+    stop("
+Sorry, GNU Java is not supported for H2O.
+Please download the latest Java SE JDK 7 from the following URL:
+http://www.oracle.com/technetwork/java/javase/downloads/jdk7-downloads-1880260.html")
+  }
+
+  if(any(grepl("Client VM", jver))) {
+    warning("
+You have a 32-bit version of Java.  H2O works best with 64-bit Java.
+Please download the latest Java SE JDK 7 from the following URL:
+http://www.oracle.com/technetwork/java/javase/downloads/jdk7-downloads-1880260.html")
+
+    # Set default max_memory to be 1g for 32-bit JVM.
+    if(is.null(max_memory)) max_memory = "1g"
+  }
+
+  if (.Platform$OS.type == "windows") {
+    slashes_fixed_ice_root = gsub("\\\\", "/", ice_root)
+  }
+  else {
+    slashes_fixed_ice_root = ice_root
+  }
+
+  # Compose args
+  mem_args <- c()
+  if(!is.null(min_memory)) mem_args <- c(mem_args, paste("-Xms", min_memory, sep=""))
+  if(!is.null(max_memory)) mem_args <- c(mem_args, paste("-Xmx", max_memory, sep=""))
+
+  args <- mem_args
+  if(assertion) args <- c(args, "-ea")
+  args <- c(args, "-jar", jar_file)
+  args <- c(args, "-name", "H2O_started_from_R")
+  args <- c(args, "-ip", "127.0.0.1")
+  args <- c(args, "-port", "54321")
+  args <- c(args, "-ice_root", slashes_fixed_ice_root)
+  if(nthreads > 0) args <- c(args, "-nthreads", nthreads)
   if(beta) args <- c(args, "-beta")
   if(!is.null(license)) args <- c(args, "-license", license)
 
   cat("\n")
   cat(        "Note:  In case of errors look at the following log files:\n")
-  cat(sprintf("           %s\n", stdout))
-  cat(sprintf("           %s\n", stderr))
+  cat(sprintf("    %s\n", stdout))
+  cat(sprintf("    %s\n", stderr))
   cat("\n")
-  system2(command, c("-version"))
+
+  # Print a java -version to the console
+  system2(command, c(mem_args, "-version"))
   cat("\n")
+
+  # Run the real h2o java command
   rc = system2(command,
                args=args,
                stdout=stdout,
@@ -211,29 +329,19 @@ h2o.clusterStatus <- function(client) {
 .h2o.getTmpFile <- function(type) {
   if(missing(type) || !type %in% c("stdout", "stderr", "pid"))
     stop("type must be one of 'stdout', 'stderr', or 'pid'")
-  
+
   if(.Platform$OS.type == "windows") {
-    default_path <- paste("C:", "TMP", sep = .Platform$file.sep)
-    if(file.exists(default_path))
-      tmp_path <- default_path
-    else if(file.exists(paste("C:", "TEMP", sep = .Platform$file.sep)))
-      tmp_path <- paste("C:", "TEMP", sep = .Platform$file.sep)
-    else if(file.exists(Sys.getenv("APPDATA")))
-      tmp_path <- Sys.getenv("APPDATA")
-    else
-      stop("Error: Cannot log Java output. Please create the directory ", default_path, ", ensure it is writable, and re-initialize H2O")
     usr <- gsub("[^A-Za-z0-9]", "_", Sys.getenv("USERNAME"))
   } else {
-    tmp_path <- paste(.Platform$file.sep, "tmp", sep = "")
     usr <- gsub("[^A-Za-z0-9]", "_", Sys.getenv("USER"))
   }
-  
+
   if(type == "stdout")
-    paste(tmp_path, paste("h2o", usr, "started_from_r.out", sep="_"), sep = .Platform$file.sep)
+    paste(tempdir(), paste("h2o", usr, "started_from_r.out", sep="_"), sep = .Platform$file.sep)
   else if(type == "stderr")
-    paste(tmp_path, paste("h2o", usr, "started_from_r.err", sep="_"), sep = .Platform$file.sep)
+    paste(tempdir(), paste("h2o", usr, "started_from_r.err", sep="_"), sep = .Platform$file.sep)
   else
-    paste(tmp_path, paste("h2o", usr, "started_from_r.pid", sep="_"), sep = .Platform$file.sep)
+    paste(tempdir(), paste("h2o", usr, "started_from_r.pid", sep="_"), sep = .Platform$file.sep)
 }
 
 .h2o.startedH2O <- function() {
@@ -278,13 +386,25 @@ h2o.clusterStatus <- function(client) {
 }
 
 .h2o.downloadJar <- function(branch, version, overwrite = FALSE) {
-  # if(missing(branch)) branch <- packageDescription("h2o")$Branch
-  if(missing(branch))
-    branch <- readLines(paste(.h2o.pkg.path, "branch.txt", sep = .Platform$file.sep))
-  if(missing(version)) version <- packageVersion("h2o")[1,4]
+  if (is.null(.h2o.pkg.path)) {
+    pkg_path = dirname(system.file(".", package = "h2o"))
+  } else {
+    pkg_path = .h2o.pkg.path
+  }
+
+  if (missing(branch)) {
+    branchFile = paste(pkg_path, "branch.txt", sep = .Platform$file.sep)
+    branch <- readLines(branchFile)
+  }
+
+  if (missing(version)) {
+    buildnumFile = paste(pkg_path, "buildnum.txt", sep = .Platform$file.sep)
+    version <- readLines(buildnumFile)
+  }
+
   if(!is.logical(overwrite)) stop("overwrite must be TRUE or FALSE")
   
-  dest_folder <- paste(.h2o.pkg.path, "java", sep = .Platform$file.sep)
+  dest_folder <- paste(pkg_path, "java", sep = .Platform$file.sep)
   if(!file.exists(dest_folder)) dir.create(dest_folder)
   dest_file <- paste(dest_folder, "h2o.jar", sep = .Platform$file.sep)
   
@@ -329,63 +449,3 @@ h2o.clusterStatus <- function(client) {
   }
   return(dest_file)
 }
-
-#-------------------------------- Deprecated --------------------------------#
-# NB: if H2OVersion matches \.99999$ is a development version, so pull package info out of file.  yes this is a hack
-#     but it makes development versions properly prompt upgrade
-# .h2o.checkPackage <- function(myURL, silentUpgrade, promptUpgrade) {
-#   h2oWrapper.__formatError <- function(error, prefix="  ") {
-#     result = ""
-#     items = strsplit(error,"\n")[[1]];
-#     for (i in 1:length(items))
-#       result = paste(result, prefix, items[i], "\n", sep="")
-#     result
-#   }
-#   
-#   temp = postForm(paste(myURL, .h2o.__PAGE_RPACKAGE, sep="/"), style = "POST")
-#   res = fromJSON(temp)
-#   if (!is.null(res$error))
-#     stop(paste(myURL," returned the following error:\n", h2oWrapper.__formatError(res$error)))
-#   
-#   H2OVersion = res$version
-#   myFile = res$filename
-#   
-#   if( grepl('\\.99999$', H2OVersion) ){
-#     H2OVersion <- sub('\\.tar\\.gz$', '', sub('.*_', '', myFile))
-#   }
-#   
-#   # sigh. I so wish people would occasionally listen to me; R expects a version to be %d.%d.%d.%d and will ignore anything after
-#   myPackages <- installed.packages()[,1]
-#   needs_upgrade <- F
-#   if( 'h2oRClient' %in% myPackages ){
-#     ver <- unclass( packageVersion('h2oRClient') )
-#     ver <- paste( ver[[1]], collapse='.' )
-#     needs_upgrade <- !(ver == H2OVersion)
-#   }
-#   
-#   if("h2oRClient" %in% myPackages && !needs_upgrade )
-#     cat("H2O R package and server version", H2OVersion, "match\n")
-#   else if(.h2o.shouldUpgrade(silentUpgrade, promptUpgrade, H2OVersion)) {
-#     if("h2oRClient" %in% myPackages) {
-#       cat("Removing old H2O R package version", toString(packageVersion("h2oRClient")), "\n")
-#       if("package:h2oRClient" %in% search())
-#         detach("package:h2oRClient", unload=TRUE)
-#       remove.packages("h2oRClient")
-#     }
-#     cat("Downloading and installing H2O R package version", H2OVersion, "\n")
-#     install.packages("h2oRClient", repos = c(H2O = paste(myURL, "R", sep = "/"), getOption("repos")))
-#   }
-# }
-# 
-# Check if user wants to install H2O R package matching version on server
-# Note: silentUpgrade supercedes promptUpgrade
-# .h2o.shouldUpgrade <- function(silentUpgrade, promptUpgrade, H2OVersion) {
-#   if(silentUpgrade) return(TRUE)
-#   if(promptUpgrade) {
-#     ans = readline(paste("Do you want to install H2O R package version", H2OVersion, "from the server (Y/N)? "))
-#     temp = substr(ans, 1, 1)
-#     if(temp == "Y" || temp == "y") return(TRUE)
-#     else if(temp == "N" || temp == "n") return(FALSE)
-#     else stop("Invalid answer! Please enter Y for yes or N for no")
-#   } else return(FALSE)
-# }
